#ifndef AWALI_JSON_HH
#define AWALI_JSON_HH

#include<awali/sttc/misc/json.hh>
#include<vector>
#include<unordered_map>
#include<cmath>

namespace awali {


  struct js_int;
  struct js_float;
  struct js_object;

  struct js_value {
    static const unsigned OBJECT;
    static const unsigned ARRAY;
    static const unsigned INTEGER;
    static const unsigned FLOATING;
    static const unsigned STRING;
    static const unsigned BOOLEAN;
    static const unsigned _NULL;
    
    virtual unsigned type() const = 0;
    
    static js_value* parse(std::istream& i);

    virtual ~js_value() {}

    virtual js_object* object();
  };
  

  struct js_int : js_value {
    int value;
    
    js_int(int v) : value(v) {}
    
    unsigned type() const override;
    
    virtual ~js_int() {}
  };

  struct js_float : js_value {
    double value;
    
    js_float(double v) : value(v) {}
    
    unsigned type() const override;

    virtual ~js_float() {}
    
    static js_value* parse(std::istream& i); 
  };

  struct js_cst : js_value {
    int value;
    
    js_cst(int v) : value(v) {}
    
    unsigned type() const override;    
    virtual ~js_cst() {}
    static js_cst* parse(std::istream& i);  
  };

  
  struct js_string : js_value {
    std::string value;
    
    js_string(const std::string & v) : value(v) {}
    
    unsigned type() const override;
    static js_string* parse(std::istream& i);
    virtual ~js_string() {}
  };
  
  struct js_array : js_value {
    std::vector<js_value*> values;
    
    unsigned type() const override;
    static js_array* parse(std::istream& i);

    ~js_array();
  };
  
  struct js_object : js_value {
    std::unordered_map<std::string,js_value*> fields;
    
    bool has_field(const std::string& key) const;
    js_value* value(const std::string& key);
    js_object* object(const std::string& key);
    std::vector<js_value*>& vect(const std::string& key);
    int intg(const std::string& key);
    std::string& str(const std::string& key);
    unsigned type() const override;
    static js_object* parse(std::istream& i);

    js_object* object() override;
    
    ~js_object();
  };

  js_value* parse(std::istream& i);
  


}//namespace awali

#endif
