// This file is part of Awali.
// Copyright 2016-2020 Sylvain Lombardy, Victor Marsault, Jacques Sakarovitch
//
// Awali is a free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

#include <stdio.h>
#include <fstream>

#include <awali/dyn/loading/locations.hh>

#include <awali/dyn/algos/sys.hh>
#include <awali/dyn/modules/output.hh>
#include <awali/dyn/modules/context.hh>

namespace awali { namespace dyn {

    void display(automaton_t aut, const std::string& name, bool history, bool horizontal) {
    std::string filename= "/tmp/"+name+".gv";
    std::ofstream o(filename);
    internal::dot(aut, o, history, horizontal) << std::endl;
    o.close();
    system(("dotty "+filename).c_str());
  }

  void 
  pdfdisplay
  ( automaton_t aut, const std::string& name, bool history, bool horizontal)
  {
    if (loading::get_dot_binary() == "") {
      *warning_stream << "Display possibility is disabled (requires dot binary)" << std::endl;
      return;
    }
    std::string filename= "/tmp/"+name+".pdf";
    std::ofstream o(filename);
    internal::pdf(aut,o,history,horizontal);
    o.close();
    std::string command;
// __APPLE__ ,  __linux__ , __unix__ , __WIN32
#ifdef __APPLE__
    command = "open ";
#else
    command = "xdg-open ";
#endif
    system((command+filename).c_str());
  }


  automaton_t parse_automaton(std::istream& i, const std::string& fmt) {
    if (fmt=="fado")
      return internal::fado(i);
    if (!fmt.compare("grail"))
      return internal::grail(i);
    if (!fmt.compare("json") || !fmt.compare("default"))
      return internal::parse_automaton(i);

    throw std::runtime_error("Unrecognised input format: "+fmt);
  }

  
  automaton_t internal::load(const std::string& s, bool& found, 
    const std::string& fmt) {
    std::map<std::string,loading::file_loc_t> auts = loading::example_automata(true);
    std::string filename (s);

    auto it = auts.find(s);
    found = (it != auts.end());
    if (found)
      filename = it->second.dir + "/" + it->second.name+"." + it->second.ext;
    
    std::ifstream fic(filename);
    if (fic.fail ())
      throw std::runtime_error ("can not load file " + filename);
    if (found)
      return dyn::parse_automaton(fic);
    return dyn::parse_automaton(fic, fmt);
  }

  automaton_t load(const std::string& s, const std::string& fmt) {  
    bool b;
    return internal::load(s,b,fmt);
  }

  
  std::ostream& put(automaton_t aut, std::ostream& o, const std::string& format)
  {
    return (o << io::of_string(format) << aut);
  }

  void save(const automaton_t aut, const std::string& filename, const std::string& fmt) {
     std::ofstream o(filename);
     put(aut,o,fmt);
     o.close();
  }

  std::ostream *error_stream = &std::cerr;
  std::ostream *warning_stream = &std::cerr;
  
  void set_error_stream(std::ostream & o) {
    error_stream = &o;    
  }

  void set_warning_stream(std::ostream & o) {
    warning_stream = &o;    
  }

}}//end of ns awali::dyn
