// This file is part of Awali.
// Copyright 2016-2020 Sylvain Lombardy, Victor Marsault, Jacques Sakarovitch
//
// Awali is a free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

#ifndef DYN_CONTEXT_DESCRIPTION_HH
#define DYN_CONTEXT_DESCRIPTION_HH

#include <iostream>
#include <sstream>
#include <stdexcept>
#include <vector>
#include <string>
#include <memory>
#include <algorithm>
#include <awali/common/json.hh>

//Definitions and factories for
// -labelset_description
// -weightset_description
// -context_description
#include <awali/dyn/core/context-description/description-types.hh>

//Weightset descriptions are handled by object corresponding to
//each semiring
#include <awali/dyn/core/context-description/weightsets/abstract-weightset.hh>
//The simples case (B, Q, Z-max-plus, etc...) are basic_weightset
#include <awali/dyn/core/context-description/weightsets/basic-weightset.hh>
//The other cases may have specific type
#include <awali/dyn/core/context-description/weightsets/cyclic-weightset.hh>

namespace awali {
namespace dyn {
namespace context {
    //To be register in abstract_weighset, each semiring requires
    //the instanciation of a static object

    inline
    const std::vector<abstract_weightset*>& instances() {
      static basic_weightset _b_description("B","b");
      static basic_weightset _z_description("Z","z");
      static basic_weightset _q_description("Q","q");
      static basic_weightset _r_description("R","r");
      static basic_weightset _c_description("C","c");
      static basic_weightset _f2_description("F2","f2");
      static basic_weightset _zmin_description("Z-min-plus","zmin");
      static basic_weightset _zmax_description("Z-max-plus","zmax");
      static basic_weightset _rmaxp_description("R-max-prod","pmax");
      static cyclic_weightset _cyclic_description;
      static std::vector<abstract_weightset*> v{
        &_b_description,
          &_z_description,
          &_q_description,
          &_r_description,
          &_c_description,
          &_f2_description,
          &_zmin_description,
          &_zmax_description,
          &_rmaxp_description,
          &_cyclic_description};
      return v;
    }

    labelset_description oneset();

    labelset_description letterset(std::string s);

    labelset_description intletterset(int a, int b);

    labelset_description intletterset(int n);

    labelset_description wordset(std::string s);

    labelset_description nullableset(labelset_description ls1);

    labelset_description ltupleset(std::vector<labelset_description> lss);
    
    weightset_description weightset(const std::string &k);

    weightset_description wtupleset(std::vector<weightset_description> wss);

    weightset_description ratweight(context_description cd);

    context_description c_desc(labelset_description ls, 
        weightset_description ws);

    context_description parse_context(js_object* jp);

    weightset_description parse_weightset(js_object* jp);
    
    labelset_description parse_labelset(js_object* jp);

    context_description parse_context(js_object* p);

    context_description parse_context(std::istream& i);

    std::string tostring(labelset_description ls, bool dynamic);

    std::string tostring(context_description ct, bool dynamic);

    std::string tostring(weightset_description ws, bool dynamic);

    std::string tostring(context_description ct, bool dynamic);

    std::vector<std::string> all_weightset_public_static_names();

    std::string all_weightset_public_static_names_as_string();
  
}}} //end of namespaces awali::dyn::context, awali::dyn, and awali

#endif
