// This file is part of Awali.
// Copyright 2016-2020 Sylvain Lombardy, Victor Marsault, Jacques Sakarovitch
//
// Awali is a free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.


#include <cora/area.hh>


namespace awali { 
namespace cora {
namespace details {

  void 
  area_t::erase(bool refresh_scr) {
    const std::string& bl= blank(w);
    for (int x = 0; x<h; x++)
      print_at(r+x, c, bl);
    if (refresh_scr)
      refresh();
  }


  void 
  area_t::print_left(const tokens_t& tokens, size_t& y_offset, size_t& x_offset) 
  {
    token_t sep = tokens.sep;
    font_style_t fs = tokens.font_style;
    bool first = true;
    for(const token_t& token : tokens) {
      size_t size = token.size();
      tokens_t t(fs);
      if ((long)(x_offset +  size + sep.size()) >= this->w) {
        x_offset = 0;
        y_offset++;
      } 
      else if (!first)
        t << sep;
      t << token;
      t.print_at(this->r+y_offset, this->c+x_offset);
      x_offset += t.length();
      first = false;
    }
  }

/* ******************************************************** Class list_area_t */
  int 
  list_area_t::end() 
  { 
    int e = start+h-1; 
    return (e >= size) ? size-1 : e-1;
  }

  void 
  list_area_t::normalize() 
  {
    // Normalizing index;
    if (index < 0 || size == 0)
      index = 0;
    else if (index >= size)
      index = size-1;

    if (start > size - (h-1)) {
      if ((h-1)>=size)
        start = 0;
      else
        start = size-(h-1);
    }
    if (index < start)
      start = index;
    if (index == start && (index != 0)) {
      start--;
    }
    if (index > end()) {
      start = index-(h-1)+1;
      if (start < 0)
        start = 0;
    }
    if (index == end() && index != (size-1)) {
      start++;
    }
  }

  void 
  list_area_t::action_up() 
  {
    index--;
    normalize();
  }


  void 
  list_area_t::action_down()
  {
    index++;
    normalize();
  }


  void 
  list_area_t::set_size(int i) 
  {
    size = i;
    normalize();
  }
  
  void 
  list_area_t::print_title(std::string str, bool selected) 
  {
    token_t t;
    if (selected)
      t << ACS_DIAMOND << " ";
    t << str;
    if (selected)
      t << " " << (ACS_DIAMOND);
    tokens_t ss(BOLD);
    ss << t;
    std::pair<int,int> offsets = ss.center(w); 
    ss.print_at(r,c+offsets.first);
  }

//   void 
//   list_area_t::print_row(bool selected, int i, std::string str) 
//   {
//     if ((i+1-start >= h) || i < start)
//       return;
//     if (i == start && i != 0) {
//       std::stringstream ss;
//       ss << "^^^ " << i+1 << " more ^^^";
//       std::string str = ss.str();
//       center(str,w-2);
//       print_at(r+i+1-start,c+1, str);
//     } 
//     else if (i+1-start == h-1  && i != size-1) {
//       std::stringstream ss;
//       ss << "vvv " << (size-i) << " more vvv";
//       std::string str = ss.str();
//       center(str,w-2);
//       print_at(r+i+1-start, c+1, str);
//     }
//     else {
//       center(str,w-2);
//       print_at( selected && (i==index),
//                 r+i+1-start,  c+1,
//                 str );
//     }
//   }
  

//   void area_t::print(const tokens_t& tokens, size_t x_offset, size_t y_offset) {
//     attron(tokens.font_style)
//     for (const token& token : tokens) {
//       token.pri
//     }
//     attron(token.font_style)
//   }


//   void draw_from_records(bool clear = true) {
// 
//   }

  void 
  list_area_t::print_row(bool selected, int i, tokens_t& tokens, 
      int before, int after) 
  {
//     if (record)
//       row_records.push_back(row_record_t{selected,i,tokens,before,after});
    if ((i+1-start >= h) || i < start)
      return;
    if (i == start && i != 0) {
      std::stringstream ss;
      ss << "^^^ " << i+1 << " more ^^^";
      std::string str = ss.str();
      center(str,w-2);
      print_at(r+i+1-start,c+1, str);
    } 
    else if (i+1-start == h-1  && i != size-1) {
      std::stringstream ss;
      ss << "vvv " << (size-i) << " more vvv";
      std::string str = ss.str();
      center(str,w-2);
      print_at(r+i+1-start, c+1, str);
    }
    else {
//       center(str,w-2);
      if (selected && i==index) {
//         tokens.font_style = STANDOUT;
        attron(STANDOUT);
        print_at (r+i+1-start,c+1,blank(before).c_str());
        print_at (r+i+1-start,c+w-1-after,blank(after).c_str());
//         attroff(STANDOUT);
      }
      tokens.print_at(r+i+1-start,c+1+before);
      attroff(STANDOUT);
//       tokens.font_style = NO_STYLE;
//       for (size_t i = 1; i<tokens.size(); i++)
//         print_token(tokens[i]);
    }
  }

}}} // end of namespaces awali::cora::details, awali::cora, and awali.
