// This file is part of Awali.
// Copyright 2016-2020 Sylvain Lombardy, Victor Marsault, Jacques Sakarovitch
//
// Awali is a free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

#include <string>
#include <unordered_map>
#include <sstream> 
#include <initializer_list>

#include <ncurses.h>

#include <cora/bless.hh>
#include <cora/token.hh>

#ifndef CORA_AREA_HH
#define CORA_AREA_HH

namespace awali { 
namespace cora {
namespace details {
  
  
  // Structure defining a printing (square area)
  typedef struct area {
    int r; // upper left corner row
    int c; // upper left corner column
    int h; // height
    int w; // width

    // erase the area, if `refreh_scr` is `true`, refresh display after erasure
    void erase(bool refresh_scr = false);
    void print_left(const tokens_t& t) {size_t y = 0, x =0; print_left(t,y,x);}
    void print_left(const tokens_t& tokens, size_t& y_offset, 
      size_t& x_offset);
    void new_line(size_t& y_offset, size_t& x_offset, int i = 1) {
      y_offset+=i; x_offset=0;}

  } area_t;

  //Specialization of area_t for printing vertical lists
  typedef struct list_area : public area_t {
    int index = 0; // currently selected element
    int start = 0; // current first displayed element 
                   // (not 0 if too many elements to display)
    int size = 0;  // current size of the list

    // Last index printed
    int end();


    // Correct  `index` and `start` so that slected
    void normalize();

    // Action of UP_ARROW on this list;
    void action_up();
    
    // Action of DOWN_ARROW on this list;
    void action_down();

    // Notify the area of how many elements are to be displayed
    void set_size(int i);
    
    // Print the title of the list
    void print_title(std::string str, bool selected = false);
  
    // Print the given string at given row.
    // The row is given to the position in the list and not in the viewport: if
    // the row is not visible, nothing is printed.
    // The boolean `selected` tells whether the row `index` should be
    // highlighted or not.
//     void print_row(bool selected, int i, std::string str);

    void print_row(bool selected, int i, tokens_t& tokens, 
        int before = 0, int after = 0);
//         bool record = true);

//     struct row_record_t {
//       bool selected;
//       int i;
//       tokens_t tokens;
//       int before;
//       int after;
//     };
// 
//     std::vector<row_record_t> row_records;
// 
//     void clear_records() { row_records.clear(); }
//     void draw_from_records(bool clear = true);

  } list_area_t;


}}} // end of namespaces awali::cora::details, awali::cora, and awali.


#endif
